import { FolderPath, FolderName, FileOrFolderPath, FilePath, FileName, RelativeOrAbsoluteFilePath, RelativeFilePath, RelativeOrAbsoluteFolderPath } from "../common/model/Primitives";
/**
 * Extremely easy to make mistakes here, so be careful!
 *
 * (Fortunately we have unit tests for this class.)
 *
 * DO NOT USE Node's 'Path' object to perform the logic: these paths are S3-esque, so may contain "..", which may be
 * accidentally resolved if we use path libraries to do the parsing.
 */
export declare class FolderPathUtils {
    private static readonly allSlashesRegex;
    static splitExtension(fileName: FileName): [string, string];
    static combineRelativeMaybe(parentFolder: FolderPath | undefined, childPath: RelativeOrAbsoluteFilePath | RelativeOrAbsoluteFolderPath): FileOrFolderPath;
    static isAbsolute(path: RelativeOrAbsoluteFilePath | RelativeOrAbsoluteFolderPath): boolean;
    static isRelative(path: RelativeOrAbsoluteFilePath | RelativeOrAbsoluteFolderPath): boolean;
    static combine(parentFolder: FolderPath | undefined, childName: RelativeFilePath | FolderName | FileName): FileOrFolderPath;
    static getFileName(filePath: FilePath): FileName;
    static getFolderName(folderPath: FolderPath): FolderName;
    static getName(folderPath: FolderPath | FilePath): FolderName | FileName;
    static getParentFolder(folderPath: FilePath | FolderPath | FileOrFolderPath): FolderPath | undefined;
    static getAncestors(folderPath: FilePath | FolderPath | FileOrFolderPath): FolderPath[];
    static allSlashes(value: string): boolean;
    /**
     * Gets the prefix that all descendants of this folder will have, that all non-descendents don't have.
     * Useful for translating folder paths to S3 prefixes.
     */
    static getDescendentPrefix(parent: FolderPath): FolderPath;
}
