import { BigSizeInBytes } from "@upload-io/upload-shared-iso/common/model/Primitives";
import autoBind from "auto-bind";

export class BoxedArithmetic<A, B> {
  static readonly bigSizeInBytes = new BoxedArithmetic<BigSizeInBytes, bigint>({
    box: x => BigSizeInBytes(x.toString()),
    unbox: x => BigInt(x),
    unit: BigInt(0),
    doAdd: (a, b) => a + b
  });

  constructor(
    private readonly params: {
      box: (b: B) => A;
      doAdd: (l: B, r: B) => B;
      unbox: (a: A) => B;
      unit: B;
    }
  ) {
    autoBind(this); // Point-free invocations on members of this class are particularly useful, so we bind members here.
  }

  add(left: A, right: A): A {
    const { unbox, box, doAdd } = this.params;
    return box(doAdd(unbox(left), unbox(right)));
  }

  sum(values: A[]): A {
    const { unbox, box, doAdd, unit } = this.params;
    return box(values.reduce((sum, value) => doAdd(unbox(value), sum), unit));
  }
}
