import { EpochMillis } from "@upload-io/upload-shared-iso";
import { EpochDays, EpochMonths } from "@upload-io/upload-shared-iso/common/model/Primitives";

export class IsoTimeUtils {
  private static readonly millisInDay = 8.64e7;

  static toEpochDay(time: EpochMillis): EpochDays {
    return EpochDays(Math.floor(time / IsoTimeUtils.millisInDay));
  }

  static fromEpochDay(time: EpochDays): EpochMillis {
    return EpochMillis(time * IsoTimeUtils.millisInDay);
  }

  static toEpochMonth(time: EpochMillis): EpochMonths {
    const date = new Date(time);
    const year = date.getUTCFullYear();
    const epochStart = 1970;
    const epochYear = year - epochStart;
    const epochMonthsFromPreviousYears = epochYear * 12;
    const epochMonthsCurrentYear = date.getUTCMonth();
    return EpochMonths(epochMonthsFromPreviousYears + epochMonthsCurrentYear);
  }

  static fromEpochMonth(time: EpochMonths): EpochMillis {
    const epoch = new Date(0);
    const epochPlusMonths = epoch.setMonth(epoch.getMonth() + time.valueOf());
    return EpochMillis(epochPlusMonths);
  }

  static trimEpochKeys<T>(
    getEpochUnit: (time: EpochMillis) => number,
    now: EpochMillis,
    keepLastNUnits: number,
    months: {
      [epochValue: number]: T;
    }
  ): {
    [epochValue: number]: T;
  } {
    const exclusiveMin = getEpochUnit(now) - keepLastNUnits;
    return Object.fromEntries(
      Object.entries(months).filter(([epochMonthStr]) => {
        const epochValue = parseInt(epochMonthStr);
        return epochValue > exclusiveMin;
      })
    );
  }
}
